"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.
"""

# -*- coding: utf-8 -*-
from __future__ import division
from math import ceil
from helpers import convertor
from libs import base
from utilities import connection_manager
from distutils.version import LooseVersion
from constants import NOT_AVAILABLE, NOT_SUPPORTED

#from flaskapp import app
#logger = app.logger

def get_info(cluster_obj, node_list, user=None, password=None):

    info = dict()
    node_list = node_list.replace(' ', '')
    node_list = convertor.text_to_list(node_list, ',')

    for node_address in node_list:
        node_obj = cluster_obj.get_node(node_address)
        info[node_address] = dict()
        if ":" in node_address and node_obj is not None:
            ip, port = base.split_ip_port(node_address)
            # ip, port = node_address.split(":")
            sock = node_obj.get_socket_from_queue()
            node_status = base.is_node_available(ip, port, user=user, password=password, sock=sock)
            if node_status is False:
                new_sock = connection_manager.create_socket(ip, port, user=user, password=password)
                if new_sock is not None:
                    sock = new_sock
                    node_status = base.is_node_available(ip, port, user=user, password=password, sock=sock)
            #node_status = base.is_node_available(ip, port, user=user, password=password)
            node_obj.put_socket_in_queue(sock)
            if node_status is True and node_obj.get_status(user=user, password=password) !="on":
                node_obj.set_status("on")
            elif node_status is False and node_obj.get_status(
                    user=user, password=password) is not "off":
                node_obj.set_status("off")

        if node_obj is not None and node_obj.get_status(user=user, password=password) == "on":
            info[node_address]['node_status'] = 'on'
            info[node_address]['cluster_name'] = node_obj.get_cluster_name()
            if node_obj.is_valid() is True:
                info[node_address]['same_cluster'] = "True"
                info[node_address]['memory'] = node_obj.get_memory_stats()
                info[node_address]['disk'] = node_obj.get_disk_stats()
                info[node_address]['build'] = node_obj.get_build()
                stats = node_obj.get_subset_of_statistics([
                    "objects", "client_connections", "cluster_size",
                    "migrate_progress_send", "migrate_progress_recv",
                    "cluster_integrity",
                    "err_out_of_space",
                    "queue", "tsvc_queue",
                    "system_swapping",
                    "err_write_fail_prole_unknown", "stat_duplicate_operation",
                    "uptime", "system_free_mem_pct", "batch_errors",
                    "storage_defrag_records", "stat_write_errs",
                    "free-pct-memory", "free-pct-disk",
                    'cluster_visibility', "tombstones",
                    ])
                info[node_address].update(stats)

                rx, tx = calculate_migrate_stats(node_obj)
                info[node_address]['migrate_incoming_remaining'] = rx
                info[node_address]['migrate_outgoing_remaining'] = tx

                configurations = node_obj.get_subset_of_configurations([
                    "proto-fd-max", "client-fd-max", "nsup-threads",
                    "scan-priority", "storage-benchmarks",
                    "replication-fire-and-forget"
                    ])
                info[node_address].update(configurations)
            elif node_obj.is_valid() is False:
                info[node_address]['same_cluster'] = "False"
                cluster_obj.remove_node(node_address)
        else:
            info[node_address]['node_status'] = 'off'

    return info


def get_stats(cluster_obj, node_address, stats_type="all", user=None, password=None):

    #info = dict(address=node_address)
    info = dict()
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None or node_obj.is_valid() is False:
        info['node_status'] = "off"
    else:
        stats = node_obj.get_statistics(True) if stats_type == "all" else node_obj.get_modified_statistics(True)
        info.update(stats)
        configurations = node_obj.get_configurations() if stats_type == "all" else node_obj.get_modified_configurations()
        if configurations is None:
            configurations = node_obj.get_updated_configurations(user=user, password=password)
        if configurations is not None:
            info.update(configurations)
        info['node_status'] = node_obj.get_status()
    return info


def get_all_config(cluster_obj, node_address):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)
    if node_obj is None or node_obj.is_valid() is False:
        info['node_status'] = "off"
    else:
        configurations = node_obj.get_configurations()
        info.update(configurations)
        info['node_status'] = node_obj.get_status()

    return info


def change_node_status(desired_status, cluster_obj, node_address, username="", password=None, ssh_key = None, usr=None, passwd=None):

    info = dict(address=node_address)
    node_obj = cluster_obj.get_node(node_address)

    if node_obj is None:
        info['node_status'] = "off"
    else:
        node_status = node_obj.get_status(user=usr, password=passwd)
        if node_status == "off" and desired_status == "off":
            info['status'] = "Failue"
            info['error'] = "Node Already Off"
        elif node_status == "on" and desired_status == "on":
            info['status'] = "Failure"
            info['error'] = "Node Already On"
        else:
            status = node_obj.change_status(desired_status, username, password, ssh_key)
            if status['status'] == "Success" and desired_status == "on":
                node_cluster_key = node_obj.get_cluster_key(user=usr, password=passwd)
                if cluster_obj.get_cluster_key(user=usr, password=passwd) != node_cluster_key and node_cluster_key != -1:
                    node_obj.set_invalid()
                elif node_cluster_key == -1:
                    node_obj.set_unknown()
                return dict(status="Success")
            elif status['status'] == "Success" and desired_status == "off":
                node_obj.set_unknown()
                return dict(status="Success")
            else:
                info.update(status)

    return info


def get_latency(cluster_obj, node_list, user=None, password=None):

    info = dict()
    node_list = node_list.replace(' ', '')
    node_list = convertor.text_to_list(node_list, ',')

    for node_address in node_list:
        info[node_address] = dict(node_status="on")
        node = cluster_obj.get_node(node_address)
        if node is None or node.get_status(user=user, password=password) == "off" or node.is_valid() is False:
            info[node_address]['node_status'] = "off"
        else:
            info[node_address]['latency'] = node.get_latency()
            info[node_address]['node_status'] = node.get_status()

    return info


def get_latency_history(cluster_obj, node_address, start_time=None, end_time=None, user=None, password=None):

    info = dict(address=node_address, node_status="on")
    node = cluster_obj.get_node(node_address)

    if node is None or node.is_valid() is False:
        info['node_status'] = "off"
    elif node.get_status(user=user, password=password) == "off":
        info['node_status'] = "off"
        info['node_build'] = node.get_build()
        info['latency_history'] = list(node.get_latency_history(start_time, end_time))
    else:
        info['node_build'] = node.get_build()
        info['latency_history'] = list(node.get_latency_history(start_time, end_time))
        info['node_status'] = "on"

    return info


def calculate_migrate_stats(node_obj):

    migrate_tx_partitions_remaining = NOT_SUPPORTED
    migrate_rx_partitions_remaining = NOT_SUPPORTED

    build = node_obj.get_build()
    if build and LooseVersion(build) >= LooseVersion('3.7.0'):

        attrs = ['migrate-rx-partitions-remaining', 'migrate-tx-partitions-remaining']
        if LooseVersion(build) >= LooseVersion('3.9.0'):
            attrs = ['migrate_rx_partitions_remaining', 'migrate_tx_partitions_remaining']

        migrate_tx_partitions_remaining = 0
        migrate_rx_partitions_remaining = 0

        # Iterate namespaces
        for ns, ns_obj in node_obj.get_namespaces().items():
            migrate_stats = ns_obj.get_subset_of_statistics(
                attrs)

            for key, value in migrate_stats.items():
                try:
                    migrate_stats[key] = int(value)
                except ValueError:
                    migrate_stats[key] = 0

            migrate_rx_partitions_remaining += migrate_stats[attrs[0]]
            migrate_tx_partitions_remaining += migrate_stats[attrs[1]]

    return migrate_rx_partitions_remaining, migrate_tx_partitions_remaining
